unit Unit5;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs,
  Vcl.StdCtrls, Unit4,

  System.Generics.Collections, Aurelius.Types.Nullable,
  Aurelius.Engine.ObjectManager, Aurelius.Drivers.Interfaces,
  Aurelius.Engine.DatabaseManager, Aurelius.Drivers.SQLite,
  Aurelius.Comp.Connection, Aurelius.Criteria.Base,  Aurelius.Criteria.Linq,
  Aurelius.Criteria.Projections;

type
  TMainForm = class(TForm)
    mmInfo: TMemo;
    btStart: TButton;
    btStop: TButton;
    Button1: TButton;
    Button2: TButton;
    procedure btStartClick(ASender: TObject);
    procedure btStopClick(ASender: TObject);
    procedure FormCreate(ASender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
  strict private
    procedure UpdateGUI;
  end;

var
  MainForm: TMainForm;

implementation

{$R *.dfm}

uses uConnectionModule, uProduct, uObjectDataLab, uStore;

resourcestring
  SServerStopped = 'Server stopped';
  SServerStartedAt = 'Server started at ';

{ TMainForm }

procedure TMainForm.btStartClick(ASender: TObject);
begin
  ServerContainer.SparkleHttpSysDispatcher.Start;
  UpdateGUI;
end;

procedure TMainForm.btStopClick(ASender: TObject);
begin
  ServerContainer.SparkleHttpSysDispatcher.Stop;
  UpdateGUI;
end;

procedure TMainForm.Button1Click(Sender: TObject);
var
  DBManager: TDatabaseManager;
begin

  try
    try
      DBManager := TDatabaseManager.Create(uConnectionModule.DatabaseConnection.AureliusConnection.CreateConnection);
      DBManager.UpdateDatabase;

    except
      on E: System.SysUtils.Exception do
      begin
        //Logger.Error(E.Message);
      end;
    end;

  finally
    DBManager.Free;
  end;
end;

procedure TMainForm.Button2Click(Sender: TObject);
var
  Manager: TObjectManager;
  store: TStore;
  a, b, c: TProduct;
begin

  Manager := ObjectDataLab.GetManager;


  store := TStore.Create;

  a := TProduct.Create;
  b := TProduct.Create;
  c := TProduct.Create;

  try
    store.Name := 'Store';
    store.ShortName := 'store';
    Manager.Save(store);


    a.Title := 'test 1';
    a.Store := store;
    Manager.Save(a);

    b.Title := 'test 2';
    b.Store := store;
    Manager.Save(b);

    c.Title := 'test 3';
    c.Store := store;
    Manager.Save(c);


    Manager.Flush(Store);
    Manager.Flush(a);
    Manager.Flush(b);
    Manager.Flush(c);

  finally
    Manager.free;
  end;
end;

procedure TMainForm.FormCreate(ASender: TObject);
begin
  UpdateGUI;
end;

procedure TMainForm.UpdateGUI;
const
  cHttp = 'http://+';
  cHttpLocalhost = 'http://localhost';
begin
  btStart.Enabled := not ServerContainer.SparkleHttpSysDispatcher.Active;
  btStop.Enabled := not btStart.Enabled;
  if ServerContainer.SparkleHttpSysDispatcher.Active then
    mmInfo.Lines.Add(SServerStartedAt + StringReplace(
      ServerContainer.XDataServer.BaseUrl,
      cHttp, cHttpLocalhost, [rfIgnoreCase]))
  else
    mmInfo.Lines.Add(SServerStopped);
end;

end.
