unit uProduct;

interface

uses
  System.Classes, System.StrUtils, System.DateUtils, Generics.Collections,
  Bcl.Validation.Attributes, Bcl.Validation.Classes, System.SysUtils,

  // Aurelius
  Aurelius.Types.Blob, Aurelius.Types.Nullable, Aurelius.Types.Proxy,
  Aurelius.Mapping.Attributes, XData.Model.Attributes,

  //Entidades
  uStore,

  System.Variants;//, uTipoConstante;

//const
 { cTypeProduct: array[0..1] of TIRegistro =
  (
    (Id: 1; Descripcion: 'Precio de lista publicado'),
    (Id: 2; Descripcion: 'Solicitar cotizacin')
  );}

type
  // Estado de product
  [Enumeration(TEnumMappingType.emChar, 'A,D,T')]
  TStatusProduct = (Activo, Descatalogado, TemporalmenteInactivo);

  [Enumeration(TEnumMappingType.emChar, 'C,A')]
  TBadgeType = (Certificado, Acreditado);

  // Para que funcione en las referencias
  TTypeProduct = class;
  TProductAttached = class;
  TProductBadge = class;
  TProductPhoto = class;
  TMatrizProduct = class;

  [Entity, Automapping]
  [Table('Product')]
  [Id('FIdProduct', TIdGenerator.IdentityOrSequence)]
  TProduct = class
  private
    [Column('IdProduct', [TColumnProp.Required])]
    FIdProduct: integer;

    [Column('Title', [TColumnProp.Required], 100)]
    FTitle: string;

    [Column('Subtitle', [TColumnProp.Required], 250)]
    FSubtitle: string;

    [Column('Description', [TColumnProp.Required], 3000)]
    FDescription: string;

    [Column('StatusProduct', [], 1)]
    FStatusProduct: TStatusProduct;

    [ForeignKey('FK_Product2Store')]
    [JoinColumn('IdStore', [TColumnProp.Lazy, TColumnProp.Required])]
    FStore: TStore;

    [ForeignKey('FK_Product2TypeProduct')]
    [JoinColumn('IdTypeProduct', [TColumnProp.Lazy{, TColumnProp.Required}])]
    FTypeProduct: TTypeProduct;

    [ForeignKey('FK_Product2MatrizProduct')]
    [JoinColumn('IdMatrizProduct', [TColumnProp.Lazy])]
    FMatrizProduct: TMatrizProduct;

    [Column('EstimatedTerm', [], 200)]
    FEstimatedTerm: Nullable<string>;

    [Column('Memo', [], 300)]
    FMemo: Nullable<string>;

    [Column('Price', [])]
    FPrice: Nullable<Currency>;

    [Column('RequestQuote', [])]
    FRequestQuote: Nullable<Boolean>;

    [Column('Methodology', [], 300)]
    FMethodology: Nullable<string>;

    [Column('RuleProcedure', [], 300)]
    FRuleProcedure: Nullable<string>;

    [Column('TestEquipmen', [], 300)]
    FTestEquipment: string;

    // Campos detalles a otras tablas
    [ForeignJoinColumn('IdProduct', [TColumnProp.Required])]
    FProductAttached: Proxy<TList<TProductAttached>>;

    [ForeignJoinColumn('IdProduct', [TColumnProp.Required])]
    FProductPhoto: Proxy<TList<TProductPhoto>>;

    [ForeignJoinColumn('IdProduct', [TColumnProp.Required])]
    FProductBadge: Proxy<TList<TProductBadge>>;


    function GetFrontText: WideString;

    function GetProductAttached: TList<TProductAttached>;
    function GetProductPhoto: TList<TProductPhoto>;
    function GetProductBadge: TList<TProductBadge>;

    // Campos autocalculados
    function GetAccredited: Boolean;
    function GetCertified: Boolean;

  public
    constructor Create; overload;
    destructor Destroy; override;

    property IdProduct: integer read FIdProduct write FIdProduct;
    property Title: string read FTitle write FTitle;
    property Subtitle: string read FSubtitle write FSubtitle;
    property Description: string read FDescription write FDescription;

    property StatusProduct: TStatusProduct read FStatusProduct write FStatusProduct;
    property Store: TStore read FStore write FStore;
    property TypeProduct: TTypeProduct read FTypeProduct write FTypeProduct;
    property MatrizProduct: TMatrizProduct read FMatrizProduct write FMatrizProduct;

    property FrontText: WideString read GetFrontText;

    property EstimatedTerm: Nullable<string> read FEstimatedTerm write FEstimatedTerm;
    property Memo: Nullable<string> read FMemo write FMemo;
    property Price: Nullable<Currency> read FPrice write FPrice;
    property RequestQuote: Nullable<Boolean> read FRequestQuote write FRequestQuote;

    property Methodology: Nullable<string> read FMethodology write FMethodology;
    property RuleProcedure: Nullable<string> read FRuleProcedure write FRuleProcedure;
    property TestEquipment: string read FTestEquipment write FTestEquipment;

    // Archivo adjunto
    property ProductAttached: TList<TProductAttached> read GetProductAttached;
    property ProductPhoto: TList<TProductPhoto> read GetProductPhoto;
    property ProductBadge: TList<TProductBadge> read GetProductBadge;

    // Campos autocalculados
    [XDataProperty]
    property Accredited: Boolean read GetAccredited;

    [XDataProperty]
    property Certified: Boolean read GetCertified;
  end;


  [Entity, Automapping]
  [Table('MatrizProduct')]
  [Id('FIdMatrizProduct', TIdGenerator.None)]
  TMatrizProduct = class
  private
    [Column('IdMatrizProduct', [TColumnProp.Required])]
    FIdMatrizProduct: integer;

    [Column('TypeProduct', [TColumnProp.Required], 100)]
    FMatrizProduct: string;

    [ForeignKey('FK_MatrizProduct2Store')]
    [JoinColumn('IdStore', [TColumnProp.Lazy, TColumnProp.Required])]
    FStore: TStore;
  public
    property IdMatrizProduct: integer read FIdMatrizProduct write FIdMatrizProduct;
    property MatrizProduct: string read FMatrizProduct write FMatrizProduct;
    property Store: TStore read FStore write FStore;
  end;


  [Entity, Automapping]
  [Table('TypeProduct')]
  [Id('FIdTypeProduct', TIdGenerator.None)]
  TTypeProduct = class
  private
    [Column('IdTypeProduct', [TColumnProp.Required])]
    FIdTypeProduct: integer;

    [Column('TypeProduct', [TColumnProp.Required], 100)]
    FTypeProduct: string;
  public
    property IdTypeProduct: integer read FIdTypeProduct write FIdTypeProduct;
    property TypeProduct: string read FTypeProduct write FTypeProduct;
  end;


  // Archivos adjuntos
  [Entity, Automapping]
  [Table('ProductAttached')]
  [Id('FIdProductAttached', TIdGenerator.IdentityOrSequence)]
  TProductAttached = class
  private
    [Column('IdProductAttached', [TColumnProp.Required])]
    FIdProductAttached: integer;

    [Column('Description', [TColumnProp.Required], 300)]
    FDescription: string;

    function GetAttachedFile: Nullable<string>;

  public
    property IdProductAttached: integer read FIdProductAttached write FIdProductAttached;
    property Description: string read FDescription write FDescription;

    [XDataProperty]
    property AttachedFile: Nullable<string> read GetAttachedFile;
  end;


  // Acreditaciones / Certificacion cargadas en PDF
  [Entity, Automapping]
  [Table('ProductBadge')]
  [Id('FIdProductBadge', TIdGenerator.IdentityOrSequence)]
  TProductBadge = class
  private
    [Column('IdStoreBadge', [TColumnProp.Required])]
    FIdProductBadge: integer;

    [Column('BadgeType', [], 1)]
    FBadgeType: TBadgeType;

    [Column('DateTime')]
    FDateTime: TDateTime;

    [Column('Title', [], 300)]
    FTitle: string;

    [Column('Description', [], 1000)]
    FDescription: Nullable<string>;

    [Column('IssueBy', [], 200)]
    FIssueBy: string;

    function GetBadgeFile: Nullable<string>;

  public
    property IdProductBadge: integer read FIdProductBadge write FIdProductBadge;
    property BadgeType: TBadgeType read FBadgeType write FBadgeType;
    property DateTime: TDateTime read FDateTime write FDateTime;
    property Title: string read FTitle write FTitle;
    property Description: Nullable<string> read FDescription write FDescription;
    property IssueBy: string read FIssueBy write FIssueBy;

    [XDataProperty]
    property BadgeFile: Nullable<string> read GetBadgeFile;
  end;


  // Fotos de producto
  [Entity, Automapping]
  [Table('ProductPhoto')]
  [Id('FIdProductPhoto', TIdGenerator.IdentityOrSequence)]
  TProductPhoto = class
  private
    [Column('IdProductPhoto', [TColumnProp.Required])]
    FIdProductPhoto: integer;

    [Column('DateTime')]
    FDateTime: TDateTime;

    [Column('Description', [], 200)]
    FDescription: Nullable<string>;

    function GetPhotoFile: Nullable<string>;

  public
    property IdProductPhoto: integer read FIdProductPhoto write FIdProductPhoto;
    property DateTime: TDateTime read FDateTime write FDateTime;
    property Description: Nullable<string> read FDescription write FDescription;
    property PhotoFile: Nullable<string> read GetPhotoFile;
  end;



implementation

{uses
  uRepository;}


{ TProduct }

constructor TProduct.Create;
begin
  FProductAttached.SetInitialValue(TList<TProductAttached>.Create);
  FProductPhoto.SetInitialValue(TList<TProductPhoto>.Create);
  FProductBadge.SetInitialValue(TList<TProductBadge>.Create);
end;


destructor TProduct.Destroy;
begin
  FProductAttached.DestroyValue;
  FProductPhoto.DestroyValue;
  FProductBadge.DestroyValue;
  inherited;
end;


function TProduct.GetAccredited: Boolean;
var
  i, r: integer;
begin

 // Reseteo las variables
  r := 0;
  Result := False;

  // Recorro los datos detalle
  for i := 0 to FProductBadge.Value.Count - 1 do
  begin
    // Cuento si conincide
    if FProductBadge.Value[i].FBadgeType = Acreditado then
    begin
      r := r + 1;
    end;
  end;

  // AL finalizar, el valor es mayor 1, el restado es True;
  if r > 0 then
  begin
    Result := True;
  end;

end;


function TProduct.GetCertified: Boolean;
var
  i, r: integer;
begin

  // Reseteo las variables
  r := 0;
  Result := False;

  // Recorro los datos detalle
  for i := 0 to FProductBadge.Value.Count - 1 do
  begin
    // Cuento si conincide
    if FProductBadge.Value[i].FBadgeType = Certificado then
    begin
      r := r + 1;
    end;
  end;

  // AL finalizar, el valor es mayor 1, el restado es True;
  if r > 0 then
  begin
    Result := True;
  end;

end;


function TProduct.GetFrontText: WideString;
begin

 { Result :=
      '<span>' +
      '<div class="media-left">'+
      '  <img class="media-object" alt="48x48" style="width: 48px; height: 48px;" src="/files/store_' + Store.IdStore.ToString + '.png">'+
      '</div>'+
      '<div class="media-body">'+
      '  <h5 class="media-heading"><b> '+ Title +
      IfThen(Acreditacion = True, ' <span title="Acreditado" class="label label-success">Acreditado</span>', '') +
      '</b></h5>'+
      '  <p> '+ Subtitle + '</p>'+
      '</div>'+
      '</span>'; }
end;


function TProduct.GetProductAttached: TList<TProductAttached>;
begin
  Result := FProductAttached.Value;
end;


function TProduct.GetProductBadge: TList<TProductBadge>;
begin
  Result := FProductBadge.Value;
end;


function TProduct.GetProductPhoto: TList<TProductPhoto>;
begin
  Result := FProductPhoto.Value;
end;


{ TProductBadge }

function TProductBadge.GetBadgeFile: Nullable<string>;
var
  sPathFile: string;
begin
  // Obtengo el path del archivo
  //sPathFile := TRepository.GetFilePath('ProductBadge', IdProductBadge, '.pdf');

  // y si existee devuelvo el nombre del archivo, sino null
  if sPathFile <> EmptyStr then
  begin
    Result := sPathFile;
  end
  else
    begin
      Result := SNull;
    end;
end;


{ TProductPhoto }

function TProductPhoto.GetPhotoFile: Nullable<string>;
var
  sPathFile: string;
begin
  // Obtengo el path del archivo
  //sPathFile := TRepository.GetFilePath('ProductPhoto', IdProductPhoto, '.jpg');

  // y si existee devuelvo el nombre del archivo, sino null
  if sPathFile <> EmptyStr then
  begin
    Result := sPathFile;
  end
  else
    begin
      Result := SNull;
    end;
end;


{ TProductAttached }

function TProductAttached.GetAttachedFile: Nullable<string>;
var
  sPathFile: string;
begin
  // Obtengo el path del archivo
  //sPathFile := TRepository.GetFilePath('ProductAttached', IdProductAttached, '.pdf');

  // y si existee devuelvo el nombre del archivo, sino null
  if sPathFile <> EmptyStr then
  begin
    Result := sPathFile;
  end
  else
    begin
      Result := SNull;
    end;
end;


initialization
  RegisterEntity(TProduct);
  RegisterEntity(TTypeProduct);
  RegisterEntity(TProductAttached);
  RegisterEntity(TProductBadge);
end.
