unit uProductServiceImplementation;

interface

uses
  XData.Server.Module, XData.Service.Common, Generics.Collections,
  System.SysUtils, System.Classes, Aurelius.Engine.ObjectManager,
  Aurelius.Criteria.Projections, Aurelius.Criteria.Base, Aurelius.Criteria.Linq,
  uProductService, uProduct;

type
  [ServiceImplementation]
  TProductService = class(TInterfacedObject, IProductService)
  private
    function FeatureProduct: TList<TProduct>;
    function SearchProduct(Title: string): TList<TProduct>;
  end;

implementation

uses
  uObjectDataLab, uConnectionModule;


function TProductService.FeatureProduct: TList<TProduct>;
var
  Manager: TObjectManager;
begin
  // https://support.tmssoftware.com/t/how-to-order-by-random-limit-10/7653
  // Creo la conexion
  Manager := ObjectDataLab.GetManager;

  try
    // Por ahora devuelvo expandiendo desde el servidor,
    //  pero preguntar a Wagner si hay una solucion / implementacion del lado cliente
    TXDataOperationContext.Current.Request.Headers.SetValue('xdata-expand-level', '2');

    // 'DBMS_RANDOM.VALUE' ---> Solo para Oracle
    // Recupero la lista de objetos
//    Result := Manager.Find<TProduct>.Take(3)
//                                    .OrderBy(TProjections.Sql<Integer>('DBMS_RANDOM.VALUE'))
//                                    .List;

    // SQL Lite test
    Result := Manager.Find<TProduct>.Take(3).List;

  finally
    // Destruyo el la conexio
    Manager.Free;
  end;

end;


function TProductService.SearchProduct(Title: string): TList<TProduct>;
var
  Manager: TObjectManager;
  Criteria: TCriteria<TProduct>;
begin
  // Creo la conexion
  Manager := ObjectDataLab.GetManager;

  try
    // Por ahora devuelvo expandiendo desde el servidor,
    //  pero preguntar a Wagner si hay una solucion / implementacion del lado cliente
    TXDataOperationContext.Current.Request.Headers.SetValue('xdata-expand-level', '2');

    // Construyo el criterio de la busqueda (y aplico funcoines nativas de Oracle)
    Criteria := Manager.Find<TProduct>
                                 .Where(Linq
                                 .Sql('REGEXP_LIKE(' +
                                    'TRANSLATE(UPPER(A.Title), '''', ''AEIOUUAEIOUU''),' +
                                    'TRANSLATE(UPPER(''' + Title +'''), '''', ''AEIOUUAEIOUU''))'))
                                 .OrderBy('Title');

    // Ejecutar y obtener resultados
    Result := Criteria.List;

  finally
    // Destruyo el la conexio
    Manager.Free;
  end;
end;


initialization
  RegisterServiceType(TProductService);

end.
