unit uStore;

interface

uses
  System.Classes, System.StrUtils, System.DateUtils, Generics.Collections,
  Bcl.Validation.Attributes, Bcl.Validation.Classes, Aurelius.Types.Blob,
  Aurelius.Types.Nullable, Aurelius.Types.Proxy, Aurelius.Mapping.Attributes,
  System.SysUtils, System.IOUtils, XData.Model.Attributes,

  // Entidades
  uUser;


type
  TStorePhoto = class;
  TStoreBadge = class;

  [Entity, Automapping]
  [Table('Store')]
  [Id('FIdStore', TIdGenerator.IdentityOrSequence)]
  TStore = class
  private
    [Column('IdStore', [TColumnProp.Unique, TColumnProp.Required, TColumnProp.NoUpdate])]
    FIdStore: integer;

    [Column('Name', [TColumnProp.Required, TColumnProp.Unique], 100)]
    FName: string;

    [Column('ShortName', [], 20)]
    FShortName: string;

    [Column('Logotipo', [TColumnProp.Lazy])]
    FLogo: TBlob;

    [Column('LogoHeader', [TColumnProp.Lazy])]
    FLogoHeader: TBlob;

    [Column('AssetLogo', [], 100)]
    FAssetLogo: string;

    [Column('AssetLogoHeader', [], 100)]
    FAssetLogoHeader: string;

    [Column('Address', [], 200)]
    FAddress: string;

    [Column('Phone', [], 40)]
    FPhone: string;

    [Column('Whatsapp', [], 20)]
    FWhatsapp: string;

    [Column('Email', [], 200)]
    FEmail: string;

    [Column('Website', [], 200)]
    FWebsite: string;

    [Column('Facebook', [], 100)]
    FFacebook: string;

    [Column('Instagram', [], 100)]
    FInstagram: string;

    [Column('twitterX', [], 100)]
    FtwitterX: string;

    [Column('Slogan', [], 300)]
    FSlogan: string;

    [Column('BiographyText', [], 500)]
    FBiographyText: string;

    [Column('RUC', [], 10)]
    FRUC: string;

    [Column('ContactName', [], 50)]
    FContactName: string;

    [Column('ContactEmail', [], 200)]
    FContactEmail: string;

    [Column('ContactPhone', [], 20)]
    FContactPhone: string;

    [Column('AccountNumber', [], 100)]
    FAccountNumber: Nullable<string>;

    [Column('Latitude')]
    FLatitude: Nullable<Double>;

    [Column('Longitude')]
    FLongitude: Nullable<Double>;

    [ForeignJoinColumn('IdStore', [TColumnProp.Required])]
    FStorePhoto: Proxy<TList<TStorePhoto>>;

    [ForeignJoinColumn('IdStore', [TColumnProp.Required])]
    FStoreBadge: Proxy<TList<TStoreBadge>>;

    function GetStorePhoto: TList<TStorePhoto>;
    function GetStoreBadge: TList<TStoreBadge>;

  public
    constructor Create; overload;
    destructor Destroy; override;

    property IdStore: integer read FIdStore write FIdStore;
    property Name: string read FName write FName;
    property ShortName: string read FShortName write FShortName;
    property Logo: TBlob read FLogo write FLogo;
    property LogoHeader: TBlob read FLogoHeader write FLogoHeader;
    property AssetLogo: string read FAssetLogo write FAssetLogo;
    property AssetLogoHeader: string read FAssetLogoHeader write FAssetLogoHeader;
    property Address: string read FAddress write FAddress;
    property Phone: string read FPhone write FPhone;
    property Whatsapp: string read FWhatsapp write FWhatsapp;
    property Email: string read FEmail write FEmail;
    property Website: string read FWebsite write FWebsite;
    property Facebook: string read FFacebook write FFacebook;
    property Instagram: string read FInstagram write FInstagram;
    property twitterX: string read FtwitterX write FtwitterX;
    property Slogan: string read FSlogan write FSlogan;
    property BiographyText: string read FBiographyText write FBiographyText;
    property RUC: string read FRUC write FRUC;
    property ContactName: string read FContactName write FContactName;
    property ContactEmail: string read FContactEmail write FContactEmail;
    property ContactPhone: string read FContactPhone write FContactPhone;
    property AccountNumber: Nullable<string> read FAccountNumber write FAccountNumber;
    property Latitude: Nullable<Double> read FLatitude write FLatitude;
    property Longitude: Nullable<Double> read FLongitude write FLongitude;
    property StorePhoto: TList<TStorePhoto> read GetStorePhoto;
    property StoreBadge: TList<TStoreBadge> read GetStoreBadge;
  end;


  // Usuarios asociados a un store (laboratorio) determinado
  [Entity, Automapping]
  [Table('StoreUser')]
  [Id('FIdStoreUser', TIdGenerator.IdentityOrSequence)]
  TStoreUser = class
  private
    [Column('IdStoreUser', [TColumnProp.Required])]
    FIdStoreUser: integer;

    [ForeignKey('FK_StoreUser2Store')]
    [JoinColumn('IdStore', [TColumnProp.Lazy, TColumnProp.Required])]
    FStore: TStore;

    [ForeignKey('FK_StoreUser2User')]
    [JoinColumn('IdUser', [TColumnProp.Lazy, TColumnProp.Required])]
    FUser: TUser;

  public
    property IdStoreUser: integer read FIdStoreUser write FIdStoreUser;
    property Store: TStore read FStore write FStore;
    property User: TUser read FUser write FUser;
  end;


  // Fotos de laboratorio
  [Entity, Automapping]
  [Table('StorePhoto')]
  [Id('FIdStorePhoto', TIdGenerator.IdentityOrSequence)]
  TStorePhoto = class
  private
    [Column('IdStorePhoto', [TColumnProp.Required])]
    FIdStorePhoto: integer;

    [Column('DateTime')]
    FDateTime: TDateTime;

    [Column('Description', [], 200)]
    FDescription: Nullable<string>;

    function GetPhotoFile: Nullable<string>;

  public
    property IdStorePhoto: integer read FIdStorePhoto write FIdStorePhoto;
    property DateTime: TDateTime read FDateTime write FDateTime;
    property Description: Nullable<string> read FDescription write FDescription;
    property PhotoFile: Nullable<string> read GetPhotoFile;
  end;


  // Acreditaciones cargadas en PDF
  [Entity, Automapping]
  [Table('StoreBadge')]
  [Id('FIdStoreBadge', TIdGenerator.IdentityOrSequence)]
  TStoreBadge = class
  private
    [Column('IdStoreBadge', [TColumnProp.Required])]
    FIdStoreBadge: integer;

    [Column('DateTime')]
    FDateTime: TDateTime;

    [Column('Title', [], 300)]
    FTitle: string;

    [Column('Description', [], 1000)]
    FDescription: Nullable<string>;

    [Column('IssueBy', [], 200)]
    FIssueBy: string;

    function GetBadgeFile: Nullable<string>;

  public
    property IdStoreBadge: integer read FIdStoreBadge write FIdStoreBadge;
    property DateTime: TDateTime read FDateTime write FDateTime;
    property Title: string read FTitle write FTitle;
    property Description: Nullable<string> read FDescription write FDescription;
    property IssueBy: string read FIssueBy write FIssueBy;

    [XDataProperty]
    property BadgeFile: Nullable<string> read GetBadgeFile;
  end;


implementation

{uses
  uServerSettings, uRepository;}


{ TStore }

constructor TStore.Create;
begin
  FStorePhoto.SetInitialValue(TList<TStorePhoto>.Create);
  FStoreBadge.SetInitialValue(TList<TStoreBadge>.Create);
end;


destructor TStore.Destroy;
begin
  FStorePhoto.DestroyValue;
  FStoreBadge.DestroyValue;
  inherited;
end;


function TStore.GetStoreBadge: TList<TStoreBadge>;
begin
  Result := FStoreBadge.Value;
end;


function TStore.GetStorePhoto: TList<TStorePhoto>;
begin
  Result := FStorePhoto.Value;
end;


{ TStoreBadge }

function TStoreBadge.GetBadgeFile: Nullable<string>;
var
  sPathFile: string;
begin
  // Obtengo el path del archivo
  //sPathFile := TRepository.GetFilePath('StoreBadge', IdStoreBadge, '.pdf');

  // y si existee devuelvo el nombre del archivo, sino null
  if sPathFile <> EmptyStr then
  begin
    Result := sPathFile;
  end
  else
    begin
      Result := SNull;
    end;
end;


{ TStorePhoto }

function TStorePhoto.GetPhotoFile: Nullable<string>;
var
  sPathFile: string;
begin
  // Obtengo el path del archivo
  //sPathFile := TRepository.GetFilePath('StorePhoto', IdStorePhoto, '.jpg');

  // y si existee devuelvo el nombre del archivo, sino null
  if sPathFile <> EmptyStr then
  begin
    Result := sPathFile;
  end
  else
    begin
      Result := SNull;
    end;
end;


initialization
  RegisterEntity(TStore);
  RegisterEntity(TStoreUser);
end.
